/**************************************************************************
 Это пример программы для монохромного OLED экрана основанного на драйвере SSD1306 
**************************************************************************/

#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define OLED_RESET 4 // Пин сброса 

/* Создаем объект дисплея SSD1306, подключенного к I2C (выводы SDA, SCL)*/
Adafruit_SSD1306 display(OLED_RESET);

int example_delay = 2000; //задержка в миллисекундах между примерами
int step_delay = 1; //задержка в миллисекундах между шагами одного примера

#define LOGONUM     10 // Количество картинок в анимации

#define LOGO_HEIGHT   20
#define LOGO_WIDTH    20
static const unsigned char PROGMEM logo_bmp[] =
{ 
  0x00,0x00,0x0F, // ....................####
  0x00,0x00,0x0F, // ....................####
  0x3E,0x08,0x0F, // ..#####.....#.......####
  0x41,0x90,0x0F, // .#.....##..#........####
  0x60,0x5E,0x0F, // .##......#.####.....####
  0x18,0x71,0x0F, // ...##....###...#....####
  0x0F,0x80,0x8F, // ....#####.......#...####
  0x08,0x00,0x4F, // ....#............#..####
  0x10,0x00,0x4F, // ...#.............#..####
  0x10,0x00,0x4F, // ...#.............#..####
  0x10,0x00,0x4F, // ...#.............#..####
  0x10,0x00,0x4F, // ...#.............#..####
  0x10,0x00,0x4F, // ...#.............#..####
  0x08,0x00,0x4F, // ....#............#..####
  0x08,0x00,0x8F, // ....#...........#...####
  0x04,0x01,0x0F, // .....#.........#....####
  0x02,0x02,0x0F, // ......#.......#.....####
  0x01,0xFC,0x0F, // .......#######......####
  0x00,0x00,0x0F, // ....................####
  0x00,0x00,0x0F  // ....................####
};

void setup() {
  Serial.begin(9600);

  // SSD1306_SWITCHCAPVCC = генерировать внутреннее напряжение на дисплее от 3,3 В
  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) { // Инициализирует дисплей. Адрес 0x3C для 128x32
    Serial.println(F("SSD1306 ошибка инициализации"));
    for(;;); // вечный цикл — дальше ничего не делать
  }

 //Очищаем буфер
  display.clearDisplay();

  example_fill_screen(); //пример заполнения экрана цветом

  example_scroll_text();    //выводим движущийся текст

  example_draw_pixel(); //пример рисования точек

  example_draw_line();  //пример рисования линий

  example_draw_rect();  //пример рисования прямоугольников

  example_draw_circle();    // пример рисования кругов

  example_draw_triangle();  // пример рисования треугольников

  example_draw_bitmap();    // выводим картинку

  example_animate_bitmap(logo_bmp, LOGO_WIDTH, LOGO_HEIGHT); //анимируем картинку

  the_end();
}

void loop() {
}

String utf8rus(String source)
{
   int i,k;
   String target;
   unsigned char n;
   char m[2] = { '0', '\0' };

   k = source.length(); i = 0;
   while (i < k) 
   {
      n = source[i]; i++;
      if (n >= 0xC0) 
      {
         switch (n) 
         {
            case 0xD0: 
            {
               n = source[i]; i++;
               if (n == 0x81) { n = 0xA8; break; }
               if (n >= 0x90 && n <= 0xBF) n = n + 0x2F;
               break;
            }
           case 0xD1: 
            {
                n = source[i]; i++;
                if (n == 0x91) { n = 0xB8; break; }
                if (n >= 0x80 && n <= 0x8F) n = n + 0x6F;
                break;
            }
         }
      }
      m[0] = n; target = target + String(m);
   }
   return target;
}

void example_fill_screen(void) {

   display.clearDisplay();
  
  display.fillScreen(BLACK);
  /*выводим пояснительный текст*/  
  display.setTextSize(1);            
  display.setTextColor(WHITE);      
  display.setCursor(0,0);          
  display.println(utf8rus("Заливка экрана"));
  display.display();     
  delay(example_delay/2);

  display.clearDisplay();
   
  display.fillScreen(WHITE);
  /*выводим пояснительный текст*/  
  display.setTextColor(BLACK);
  display.setCursor(0,0);      
  display.println(utf8rus("Заливка экрана"));
  display.display();     
  delay(example_delay/2);
}

void example_scroll_text(void) {

  /*выводим пояснительный текст*/
  display.clearDisplay();

  display.setTextSize(1);            
  display.setTextColor(WHITE);      
  display.setCursor(0,0);          
  display.println(utf8rus("Движущийся текст"));
  display.display();
  delay(example_delay/2);

  display.clearDisplay();

  display.setTextSize(2); 
  display.setTextColor(WHITE);
  display.setCursor(10, 0);
  display.println(utf8rus("Текст"));
  display.display();     
 delay(example_delay/2);

  // Скроллинг текста в разных направлениях
  display.startscrollright(0x00, 0x0F);
 delay(example_delay/2);
  display.stopscroll();
 delay(example_delay/2);
  display.startscrollleft(0x00, 0x0F);
 delay(example_delay/2);
  display.stopscroll();
  delay(example_delay/2);
  display.startscrolldiagright(0x00, 0x07);
 delay(example_delay/2);
  display.startscrolldiagleft(0x00, 0x07);
 delay(example_delay/2);
  display.stopscroll();
  delay(example_delay/2);
}

void example_draw_pixel(void) {
  /*выводим пояснительный текст*/
  display.clearDisplay();

  display.setTextSize(1);             
  display.setTextColor(WHITE);        
  display.setCursor(0,0);            
  display.println(utf8rus("Рисуем точки"));
  display.display();
  delay(example_delay/2);

   /*выводим точки*/
  display.clearDisplay();

  int16_t i, j = display.height()/2; 

  for(i=0; i<display.width(); i+=10) {
    display.drawPixel(i, j, WHITE);
    display.display(); 
    delay(step_delay);
  }

  delay(example_delay);
}

void example_draw_line(void) {

  /*выводим пояснительный текст*/
  display.clearDisplay();

  display.setTextSize(1);            
  display.setTextColor(WHITE);      
  display.setCursor(0,0);          
  display.println(utf8rus("Рисуем линии"));
  display.display();
  delay(example_delay/2);

  int16_t i;

/*выводим вертикальные линии*/

  display.clearDisplay(); 

  for(i=0; i<display.width(); i+=8) {
    display.drawLine(i, 0, i, display.height()-1, WHITE);
    display.display(); 
    delay(step_delay);
  }

  delay(example_delay/2);

  /*выводим горизонтальные линии*/

  display.clearDisplay();
   
  for(i=0; i<display.height(); i+=4) {
    display.drawLine(0, i, display.width()-1, i, WHITE);
    display.display();
    delay(step_delay);
  }
  
  delay(example_delay/2);

  /*выводим диагональные линии*/

  display.clearDisplay();
 
  for(i=0; i<display.height(); i+=4) {
    display.drawLine(display.width()-1, 0, 0, i, WHITE);
    display.display();
    delay(step_delay);
  }
   for(i=8; i<display.width(); i+=8) {
    display.drawLine(display.width()-1, 0, i, display.height()-1, WHITE);
    display.display();
    delay(step_delay);
  }
 
  delay(example_delay/2);

  display.clearDisplay();

   for(i=0; i<display.width()-1; i+=8) {
    display.drawLine(i, 0, display.width()-1-i, display.height()-1, WHITE);
    display.display();
    delay(step_delay);
  }
   for(i=4; i<display.height()-1; i+=4) {
    display.drawLine(display.width()-1, i, 0, display.height()-1-i, WHITE);
    display.display();
    delay(step_delay);
  }
  
  delay(example_delay); 
}

void example_draw_rect(void) {

   /*выводим пояснительный текст*/
  display.clearDisplay();

  display.setTextSize(1);            
  display.setTextColor(WHITE);      
  display.setCursor(0,0);          
  display.println(utf8rus("Рисуем прямоугольники"));
  display.display();
  delay(example_delay/2);

  /*выводим прямогугольники вложенные друг в друга*/
  
 display.clearDisplay();

 for(int16_t i=0; i<display.height()/2; i+=2) {
    display.drawRect(i, i, display.width()-2*i, display.height()-2*i, WHITE);
    display.display(); 
    delay(step_delay);
  }

  delay(example_delay/2);

  /*выводим заполненные цветом прямоугольники*/  
   display.clearDisplay();

  for(int16_t i=0; i<display.width()-1; i+=16) {
    display.fillRect(i, 0, 15, display.height()-1, WHITE);
    display.display(); 
    delay(step_delay);
  }

  delay(example_delay/2);

   /*выводим скругленные прямоугольники*/
   display.clearDisplay();

   for(int16_t i=0; i<display.height()/2-2; i+=2) {
    display.drawRoundRect(i, i, display.width()-2*i, display.height()-2*i,
    display.height()/4, WHITE);
    display.display();
    delay(step_delay);
  }

  delay(example_delay/2);

   /*выводим заполненные цветом прямоугольники*/  
   display.clearDisplay();

  for(int16_t i=0; i<display.width()-1; i+=16) {
    display.fillRoundRect(i, 0, 15, display.height()-1, display.height()/4, WHITE);
    display.display(); 
    delay(step_delay);
  }

  delay(example_delay);
}

void example_draw_circle(void) {

  /*выводим пояснительный текст*/
  display.clearDisplay();

  display.setTextSize(1);            
  display.setTextColor(WHITE);      
  display.setCursor(0,0);          
  display.println(utf8rus("Рисуем круги"));
  display.display();
  delay(example_delay/2);

  /*выводим круги*/
  display.clearDisplay();

  for(int16_t i=0; i<display.width()-1; i+=display.height()-1) {
    display.drawCircle(i, display.height()/2, display.height()/2-1, WHITE);
    display.display();
    delay(step_delay);
  }

  delay(example_delay/2);

  /*выводим заполненные круги*/
  display.clearDisplay();

  for(int16_t i=0; i<display.width()-1; i+=display.height()-1) {
    display.fillCircle(i, display.height()/2, display.height()/2-1, WHITE);
    display.display();
    delay(step_delay);
  }

  delay(example_delay);
}


void example_draw_triangle(void) {

  /*выводим пояснительный текст*/
  display.clearDisplay();

  display.setTextSize(1);            
  display.setTextColor(WHITE);      
  display.setCursor(0,0);          
  display.println(utf8rus("Рисуем треугольники"));
  display.display();
  delay(example_delay/2);

  /*выводим треугольники*/
  display.clearDisplay();

  for(int16_t i=0; i<display.width()-1; i+=display.height()-1) {
    display.drawTriangle(
    i, 0,
    i - (display.height()/2), display.height()-1,
    i + (display.height()/2), display.height()-1, WHITE);
    display.display();
    delay(step_delay);
  }

  delay(example_delay/2);

   /*выводим треугольники заполненные цветом*/

   display.clearDisplay();

   for(int16_t i=0; i<display.width()-1; i+=display.height()-1) {
    display.fillTriangle(
    i, 0,
    i - (display.height()/2), display.height()-1,
    i + (display.height()/2), display.height()-1, WHITE);
    display.display();
    delay(step_delay);
  }

  delay(example_delay);
}

void example_draw_bitmap(void) {
  /*выводим пояснительный текст*/
  display.clearDisplay();

  display.setTextSize(1);            
  display.setTextColor(WHITE);      
  display.setCursor(0,0);          
  display.println(utf8rus("Рисуем картинку"));
  display.display();
  delay(example_delay/2);

  /*выводим картинку*/
  
  display.clearDisplay();
  display.drawBitmap(
    (display.width()  - LOGO_WIDTH ) / 2,
    (display.height() - LOGO_HEIGHT) / 2,
    logo_bmp, LOGO_WIDTH, LOGO_HEIGHT, 1);
  display.display();
  delay(example_delay*2);
}

#define XPOS   0 // Индексирует в массив 'icons' в функции ниже
#define YPOS   1
#define DELTAY 2

void example_animate_bitmap(const uint8_t *bitmap, uint8_t w, uint8_t h) {

   /*выводим пояснительный текст*/
  display.clearDisplay();

  display.setTextSize(1);            
  display.setTextColor(WHITE);      
  display.setCursor(0,0);          
  display.println(utf8rus("Анимация картинки"));
  display.display();
  delay(example_delay/2);

  /*выводим анимацию*/
  
  int8_t i, f, icons[LOGONUM][3];

  // Инициируем позиции картинок
  for(f=0; f< LOGONUM; f++) {
    icons[f][XPOS]   = random(1 - LOGO_WIDTH, display.width());
    icons[f][YPOS]   = -LOGO_HEIGHT;
    icons[f][DELTAY] = random(1, 6);
    Serial.print(F("x: "));
    Serial.print(icons[f][XPOS], DEC);
    Serial.print(F(" y: "));
    Serial.print(icons[f][YPOS], DEC);
    Serial.print(F(" dy: "));
    Serial.println(icons[f][DELTAY], DEC);
  }

  for(i=0;i<50;i++) { //цикл анимации 50 итераций
    display.clearDisplay(); // Clear the display buffer

    // Рисуем каждую картинку:
    for(f=0; f< LOGONUM; f++) {
      display.drawBitmap(icons[f][XPOS], icons[f][YPOS], bitmap, w, h, WHITE);
    }

    display.display(); // Вывести буфер на экран
    delay(200);        // пауза

    // обновляем координаты каждой картинки...
    for(f=0; f< LOGONUM; f++) {
      icons[f][YPOS] += icons[f][DELTAY];
      // если картинка находится внизу экрана...
      if (icons[f][YPOS] >= display.height()) {
        // Снова инициализируем в случайную позицию
        icons[f][XPOS]   = random(1 - LOGO_WIDTH, display.width());
        icons[f][YPOS]   = -LOGO_HEIGHT;
        icons[f][DELTAY] = random(1, 6);
      }
    }
  }
}

void the_end(void) {
  /*выводим пояснительный текст*/
  display.clearDisplay();

  display.setTextSize(1);            
  display.setTextColor(WHITE);      
  display.setCursor(0,0);          
  display.println(utf8rus("Спасибо за внимание!"));
  display.display();
}
